#include "mainwindow.h"
#include "mdiary.h"

enum {
	COL_TITLE = 0,
	COL_DATE,
	COL_TAGS,
	COL_TIMESTAMP,
	COL_COUNT
};

static void mainWindow_list_add_entry(struct mainWindow *mainWindow, struct mdiary_entry *mdiary_entry) {
	GtkTreeIter iter;
	gchar *date_text;

	date_text = g_date_time_format(mdiary_entry->date, "%A, %e %B %Y %R");

	gtk_list_store_append(mainWindow->entryListStore, &iter);
	gtk_list_store_set(mainWindow->entryListStore, &iter,
			   COL_TITLE, mdiary_entry->title,
			   COL_DATE, date_text,
			   COL_TAGS, mdiary_entry->tags,
			   COL_TIMESTAMP, g_date_time_to_unix(mdiary_entry->date),
			   -1);

	g_free(date_text);
}

static void mainWindow_configure_treeView(struct mainWindow *mainWindow)
{
	GtkCellRenderer *renderer;
	GtkTreeSortable *sortable;
	GtkTreeViewColumn *col;

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Title", renderer,
						    "text", COL_TITLE, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_TITLE);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_TITLE);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Date", renderer,
						    "text", COL_DATE, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_DATE);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_TIMESTAMP);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(mainWindow->entryListView, -1, "Tags", renderer,
						    "text", COL_TAGS, NULL);
	col = gtk_tree_view_get_column(mainWindow->entryListView, COL_TAGS);
	gtk_tree_view_column_set_resizable(col, 1);
	gtk_tree_view_column_set_sort_column_id(col, COL_TAGS);

	sortable = GTK_TREE_SORTABLE(mainWindow->entryListStore);
	gtk_tree_sortable_set_sort_column_id(sortable, COL_DATE, GTK_SORT_DESCENDING);

	/*
	 * Test entries for debugging only
	 */

	struct mdiary_entry test;
	test.title = "Hello";
	test.tags = "world";
	test.date = g_date_time_new_local(2017, 11, 21, 02, 03, 0);
	mainWindow_list_add_entry(mainWindow, &test);
	g_date_time_unref(test.date);

	test.title = "Entry";
	test.tags = "Tag, B, C";
	test.date = g_date_time_new_local(2016, 10, 4, 5, 34, 0);
	mainWindow_list_add_entry(mainWindow, &test);
	g_date_time_unref(test.date);

	test.title = "A Third One for Testing";
	test.tags = "Yey, stuff, works";
	test.date = g_date_time_new_local(2016, 12, 22, 1, 55, 0);
	mainWindow_list_add_entry(mainWindow, &test);
	g_date_time_unref(test.date);
}

static void mainWindow_connect_signals(struct mainWindow *mainWindow)
{
	g_signal_connect(mainWindow->dateStart,
			 "focus-in-event",
			 mainWindow_showPopover,
			 mainWindow);
	g_signal_connect(mainWindow->dateEnd,
			 "focus-in-event",
			 mainWindow_showPopover,
			 mainWindow);
	g_signal_connect(mainWindow->calendarRange,
			 "day-selected",
			 mainWindow_calendarSelected,
			 mainWindow);
}

struct mainWindow *mainWindow_new()
{
	GtkBuilder *builder;
	struct mainWindow *mainWindow;

	builder = gtk_builder_new_from_file("glade/mainWindow.glade");

	mainWindow = malloc(sizeof(struct mainWindow));
	mainWindow->mainWindow            = GTK_WIDGET(gtk_builder_get_object(builder, "mainWindow"));
	mainWindow->buttonChooseWorkspace = GTK_WIDGET(gtk_builder_get_object(builder, "buttonChooseWorkspace"));
	mainWindow->searchEntry           = GTK_WIDGET(gtk_builder_get_object(builder, "searchEntry"));
	mainWindow->dateStart             = GTK_WIDGET(gtk_builder_get_object(builder, "dateStart"));
	mainWindow->dateEnd               = GTK_WIDGET(gtk_builder_get_object(builder, "dateEnd"));
	mainWindow->entryText             = GTK_WIDGET(gtk_builder_get_object(builder, "entryText"));
	mainWindow->popoverDate           = GTK_WIDGET(gtk_builder_get_object(builder, "popoverDate"));
	mainWindow->calendarRange         = GTK_WIDGET(gtk_builder_get_object(builder, "calendarRange"));
	mainWindow->entryListView         = GTK_TREE_VIEW(gtk_builder_get_object(builder, "entryListView"));
	mainWindow->entryListStore        = GTK_LIST_STORE(gtk_builder_get_object(builder, "entryListStore"));

	gtk_builder_connect_signals(builder, NULL);

	g_object_unref(G_OBJECT(builder));

	mainWindow_configure_treeView(mainWindow);
	mainWindow_connect_signals(mainWindow);

	return mainWindow;
}

void mainWindow_chooseWorkspaceClicked(GtkWidget *widget, gint event, gpointer user_data)
{
	printf("Choose workspace.\n");
	fflush(stdout);
}

void mainWindow_showPopover(GtkWidget *widget,  GdkEvent *event, gpointer user_data)
{
	struct mainWindow *mainWindow = (struct mainWindow *)user_data;

	mainWindow->selectedDateEntry = widget;

	if (!gtk_widget_get_visible(mainWindow->popoverDate))
		gtk_popover_popup(GTK_POPOVER(mainWindow->popoverDate));
	gtk_popover_set_relative_to(GTK_POPOVER(mainWindow->popoverDate), widget);
}

void mainWindow_calendarSelected(GtkWidget *widget, gpointer user_data)
{
	struct mainWindow *mainWindow = (struct mainWindow *)user_data;
	guint year, month, day;
	gchar *text;

	gtk_calendar_get_date(GTK_CALENDAR(mainWindow->calendarRange), &year, &month, &day);
	text = g_strdup_printf("%d-%02d-%02d", year, month, day);
	gtk_entry_set_text(GTK_ENTRY(mainWindow->selectedDateEntry), text);
	g_free(text);

	gtk_popover_popdown(GTK_POPOVER(mainWindow->popoverDate));
}
